#include <stdio.h>
#include <stdlib.h>
#include <pam.h>

/* pgmtosli.   This program reads a PGM file from stdin and writes a
   text array in stdout with the pixel values separated as spaces.
   Each image row is a text line. The columns are separated by spaces.
   There is no header or footer describing the size of the image.

   This format suits the fscanfMat(string Filename) function of scilab */

int main(int argn, char *argv[])
{
	struct pam inpam;
	tuple *tuplerow;
	int row;

	pnm_init(&argn, argv);

	pnm_readpaminit(stdin, &inpam, PAM_STRUCT_SIZE(tuple_type));
	if (inpam.depth != 1)
	{
		fprintf(stderr,"The number of planes should be 1.\n");
		return 1;
	}

	/* Memory allocated for the row */
	tuplerow = pnm_allocpamrow(&inpam);
	if (tuplerow == NULL) 
	{
		fprintf(stderr,"Error allocating memory for a row.\n");
		return 2;
	}

	for (row = 0; row < inpam.height; row++) {
		unsigned int column;
		pnm_readpamrow(&inpam, tuplerow);
		for (column = 0; column < inpam.width; ++column) {
			if (column == inpam.width - 1)
				printf("%lu\n", tuplerow[column][0]);
			else
				printf("%lu ", tuplerow[column][0]);
		}
	}

	pnm_freepamrow(tuplerow);

	return 0;
}
